@@ -65,18 +65,9 @@
 // Define if you have the nanosleep() function.
 // #undef HAVE_NANOSLEEP
 
-// Define if you can safely include both <sys/time.h> and <time.h>.
-#define TIME_WITH_SYS_TIME
-
 // Define if you can safely include both <sys/time.h> and <sys/select.h>.
 // #undef SYS_SELECT_WITH_SYS_TIME
 
-// Define as the return type of signal handlers (int or void).
-#define RETSIGTYPE void
-
-// Define as the command at the end of signal handlers ("" or "return 0;").
-#define SIGRETURN return
-
 // Define if struct sigcontext is present
 #define HAVE_SIGCONTEXT
 
@@ -119,7 +110,6 @@
 #define HAVE_SETENV
 #define HAVE_SETJMP_H
 #define HAVE_MATH_H
-#define HAVE_FLOAT_FUNCS
 #define HAVE_GETTIMEOFDAY
 #define HAVE_PWD_H
 #define HAVE_NETDB_H
